// ***************************************
// Plugin Data holding class for SourceBoost IDE
// 
// Author(s): David Hobday
//
// Copyright (C) 2003-2004 Pavel Baranov
// Copyright (C) 2003-2004 David Hobday
// 
// This code may be freely used to develop
// Plugins for the SourceBoost IDE
// ***************************************

#ifndef _PLUGINDATA_H_
#define _PLUGINDATA_H_

#include "resource.h"		// main symbols
#include "IoPin.h"
#include "clktimer.h"

class CPluginData
{
public:

	///////////////////////////////////////////////////////////
	// Functions called in sync with sim thread so no locking
	// required. No locking mean faster execution!
	///////////////////////////////////////////////////////////
	CPluginData();
	~CPluginData();

	// Theses functions all get called as the result of a call back by the SourceBoost IDE
	void Stop(); // called when plugin is no longer displayed in active debugger
	void Start(); // called when plugin is displayed in active debugger
	void SaveState( ostream& os ); // called before each debugger step forward, to allow current state to be saved
	void LoadState( istream& is ); // called when step backward is invoked, to allow previous state to be restored
	void Step( short sDirection ); // called every step forward or backward
	void Instr( long clk ); // called after every processor instruction fetch execute cycle is performed
	void Clk( long clk ); // called after every single processor clock cycle
	void Reset(); // called when simulator is reset

	
	///////////////////////////////////////////////////////////
	// Functions that will be called by GUI thread
	// lock and unlock essential
	///////////////////////////////////////////////////////////

	// This function gets called as the result of a call back by the SourceBoost IDE
	void SetClkRate( long clockRate_Hz ); // called by IDE to set simulated clock rate
	
	// Application specific functions.
	// These functions are call by the PluginView Code	
	void GetPinName( CString& pinName ); 
	void SetPinName( const char* pinName ); 
	void Connect();
	void Disconnect();

	void SetV( int Vmv ); // set voltage in mv
	int GetV(); // get current voltage in mv

private:		
	///////////////////////////////////////////////////////////
	// Lock and Unlock functions,
	// These functions effectively pause sim thread at the same
	// point making sim access safe and predictable :)
	///////////////////////////////////////////////////////////

	void Lock();
	void Unlock();

	///////////////////////////////////////////////////////////
	// Internal functions, only called from function that 
	// apply locks if necessary
	///////////////////////////////////////////////////////////
	// Application specific functions.
	void PinUpdate();
	
	int m_VoutmV;
	CString m_pinName;

	CAnalogIoPin m_pin;
};



#endif // _PLUGINDATA_H_
