// ***************************************
// Plugin Hookup to IDE for SourceBoost IDE
// 
// These functions provide the interface to
// the IDE - this file should generally not 
// need modification.

// Author(s): David Hobday
//
// Copyright (C) 2003-2004 Pavel Baranov
// Copyright (C) 2003-2004 David Hobday
// All Rights Reserved                                    
// ***************************************

#include "stdafx.h"
#include "PluginApp.h"


#define _PLUGIN_FUNC_PTRS_DECLARE_
#include "pluginAPI.h"

#define _PLUGIN_FUNC_PTRS_RESOLVE_
#include "pluginAPI.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CPluginApp app;

PLUGIN_API int plgVersion( void )
{
	// called by gui thread
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	int ret = PLUGIN_API_VERSION; // plugin requires version compatible interface
	return ret;

}


PLUGIN_API HWND plgCreate( HWND parent, void* (*SupplyFuncPtr)(const char*) )
{	
	// called by gui thread - sim thread not running
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	// resolve function pointers to IDE services
	if ( ! ResolveFuncPtrs( SupplyFuncPtr ) )
		return NULL; 
		
	return app.CreateView( parent ); // create plugin window
}

PLUGIN_API BOOL plgFree( void )
{	
	// called by gui thread - sim thread not running
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return app.DestroyView();
}

PLUGIN_API BOOL plgStart( void )
{
	// called by gui thread - sim thread not running
	app.data.Start();
	return true;
}

PLUGIN_API void plgStop( void )
{
	// called by gui thread - sim thread not running
	app.data.Stop();
}


PLUGIN_API LPCTSTR plgName( void )
{
	// called by gui thread - sim thread not running
	return app.Name();	
}


PLUGIN_API void plgClockRate( long clockRate_Hz )
{
	// called by gui thread
	app.data.SetClkRate( clockRate_Hz );
}

PLUGIN_API void plgConfigure( HWND parent )
{
	// called by gui thread
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	
	app.Configure( parent );
}

PLUGIN_API int plgSave( BYTE *buf )
{	
	// called by gui thread - sim thread not running
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	ostrstream ss;
	app.view->Save( ss );
	
	if ( buf )
	{
		memcpy( (char*)buf, ss.str(), ss.pcount() );
		ss.rdbuf()->freeze( false ); // release buffer
	}

	return ss.pcount();
}

PLUGIN_API BOOL plgLoad( BYTE *buf, int len )
{
	// called by gui thread - sim thread not running
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	istrstream ss( (char*)buf, len );
	app.view->Load( ss );

	return ss.good();
}

PLUGIN_API int plgSaveState( BYTE *buf )
{
	// called by gui thread when sim not running
	// and called by sim thread
	ostrstream ss;

	app.data.SaveState( ss );

	if ( buf )
	{
		memcpy( (char*)buf, ss.str(), ss.pcount() );
		ss.rdbuf()->freeze( false ); // release buffer
	}

	return ss.pcount();	
}

PLUGIN_API BOOL plgLoadState( BYTE *buf, int len )
{
	// called by sim thread
	istrstream ss( (char*)buf, len );

	app.data.LoadState( ss );	

	return ss.good();
}

PLUGIN_API void plgReset()
{
	// called by sim thread
	app.data.Reset();
}

PLUGIN_API void plgStep( short sDirection )
{
	// called by gui thread before sim thread running 
	// and called by sim thread
	app.data.Step( sDirection );
}


PLUGIN_API void plgClock( long clk, void*pParam )
{
	// called by sim thread
	app.data.Clk( clk );
}

PLUGIN_API void plgInstr( long clk, void*pParam )
{
	// called by sim thread
	app.data.Instr( clk );
}
